import { useState, useEffect, useMemo } from 'react';
import NsAccordionItem from './NsAccordionItem';
import { NsAccordionProps } from './types';
import { StyledAccordionWrapper } from './ToggleSection.styled';

/**
 * NsAccordion can be used to show a list of items with title and body
 * it accepts a `boolean` prop `allowMultipleExpanded` which when true,
 * multiple sections of accordion can remain open at a time
 *
 * we can define certain section to remain open by default using `isDefaultOpen` in the
 * contentArray item
 * if there are multiple items available for default open, but `allowMultipleExpanded`
 * is false then only first item of default open is made open
 */
const NsAccordion = ({
    additionalProps,
    allowMultipleExpanded,
    contentArray,
    variant,
    headerTypographyVariant,
    closedIconName,
    openedIconName,
}: NsAccordionProps) => {
    const defaultOpenIds = useMemo(() => {
        return contentArray
            .map((content, index) => (content?.isDefaultOpen ? index : null))
            .filter((e) => typeof e === 'number') as number[];
    }, [contentArray]);
    const [expandedIds, setExpandedIds] = useState<Array<number>>(defaultOpenIds);

    const toggleOpen = (sectionNumber: number, isExpanded: boolean) => {
        if (allowMultipleExpanded) {
            if (isExpanded) {
                setExpandedIds(expandedIds.filter((existingNumber) => existingNumber !== sectionNumber));
            } else {
                setExpandedIds([...expandedIds, sectionNumber]);
            }
        } else {
            if (isExpanded) {
                setExpandedIds([]);
            } else {
                setExpandedIds([sectionNumber]);
            }
        }
    };
    useEffect(() => {
        setExpandedIds(allowMultipleExpanded ? defaultOpenIds : [defaultOpenIds[0]]);
    }, [allowMultipleExpanded, defaultOpenIds]);

    return (
        <StyledAccordionWrapper
            variant={variant}
            className="ns-accordion"
            data-testid={additionalProps?.testid ?? 'ns-accordion-testid'}
        >
            {contentArray.map((content, index) => {
                const isOpen = expandedIds.includes(index);
                return (
                    <NsAccordionItem
                        variant={variant}
                        headerTypographyVariant={headerTypographyVariant}
                        closedIconName={closedIconName}
                        openedIconName={openedIconName}
                        key={index}
                        isOpen={isOpen}
                        toggleOpen={() => toggleOpen(index, isOpen)}
                        content={content?.body}
                        title={content?.title}
                    />
                );
            })}
        </StyledAccordionWrapper>
    );
};

export default NsAccordion;
