import React from 'react';
import { render, fireEvent, screen } from '@/test-utils/index';
import { AccordionVariant, NsAccordion, NsAccordionProps } from '@/common/index';

const generateContentArray = (count: number) => {
    const arr = [] as NsAccordionProps['contentArray'];
    for (let i = 1; i <= count; i++) {
        arr.push({
            title: `Title ${i}`,
            body: `Content ${i}`,
            isDefaultOpen: true,
        });
    }
    return arr;
};

describe('NsAccordion', () => {
    const contentArray = generateContentArray(3);
    const defaultProps = {
        additionalProps: {
            testid: 'test-accordion',
        },
        allowMultipleExpanded: false,
        contentArray,
        variant: AccordionVariant.Basic,
    };

    it('renders without error', () => {
        const { getByTestId } = render(<NsAccordion {...defaultProps} />);
        const accordion = getByTestId(defaultProps.additionalProps.testid);
        expect(accordion).toBeInTheDocument();
    });

    it(`only first section should be visible when allowMultipleExpanded prop is false and multiple items have
		isDefaultOpen true`, () => {
        const { getAllByText } = render(<NsAccordion {...defaultProps} />);
        const [title1, title2, title3] = getAllByText(/Title /i);

        expect(title1.closest('header')).toHaveAttribute('aria-expanded', 'true');
        expect(title2.closest('header')).toHaveAttribute('aria-expanded', 'false');
        expect(title3.closest('header')).toHaveAttribute('aria-expanded', 'false');
        expect(screen.queryByText(/Content 1/i)).not.toBeNull();
        expect(screen.queryByText(/Content 2/i)).toBeNull();
        expect(screen.queryByText(/Content 3/i)).toBeNull();
    });

    it('expands second section only when allowMultipleExpanded prop is false and second title is clicked', async () => {
        const { getAllByText } = render(<NsAccordion {...defaultProps} />);
        const [title1, title2, title3] = getAllByText(/Title /i);
        const header = title2.closest('header');
        expect(header).toHaveAttribute('aria-expanded', 'false');
        if (header) {
            fireEvent.click(header);
            expect(title1.closest('header')).toHaveAttribute('aria-expanded', 'false');
            expect(title2.closest('header')).toHaveAttribute('aria-expanded', 'true');
            expect(title3.closest('header')).toHaveAttribute('aria-expanded', 'false');
            expect(screen.queryByText(/Content 1/i)).toBeNull();
            expect(screen.queryByText(/Content 2/i)).not.toBeNull();
            expect(screen.queryByText(/Content 3/i)).toBeNull();
        }
    });

    it('expands multiple sections by default when allowMultipleExpanded prop is true', () => {
        const contentArray = generateContentArray(3);
        contentArray[2].isDefaultOpen = false;
        const defaultProps = {
            additionalProps: {
                testid: 'test-accordion',
            },
            allowMultipleExpanded: false,
            contentArray,
            variant: AccordionVariant.Basic,
        };
        const { getAllByText } = render(<NsAccordion {...defaultProps} allowMultipleExpanded />);
        const [title1, title2, title3] = getAllByText(/Title /i);

        expect(title1.closest('header')).toHaveAttribute('aria-expanded', 'true');
        expect(title2.closest('header')).toHaveAttribute('aria-expanded', 'true');
        expect(title3.closest('header')).toHaveAttribute('aria-expanded', 'false');
        expect(screen.queryByText(/Content 1/i)).not.toBeNull();
        expect(screen.queryByText(/Content 2/i)).not.toBeNull();
        expect(screen.queryByText(/Content 3/i)).toBeNull();

        fireEvent.click(title3);

        expect(title1.closest('header')).toHaveAttribute('aria-expanded', 'true');
        expect(title2.closest('header')).toHaveAttribute('aria-expanded', 'true');
        expect(title3.closest('header')).toHaveAttribute('aria-expanded', 'true');
        expect(screen.queryByText(/Content 1/i)).not.toBeNull();
        expect(screen.queryByText(/Content 2/i)).not.toBeNull();
        expect(screen.queryByText(/Content 3/i)).not.toBeNull();
    });
});
